/* Radio.Unfix - common.c - Common Functions */

#include "radio.h"

extern struct Radio radio;

void dolog(int priority, const char *format, ...)
{
	va_list ap;
	va_start(ap, format);
	if (radio.c_Daemonize) vsyslog(LOG_LOCAL7|priority, format, ap);
	else vprintf(format, ap);
	va_end(ap);
}

void query(int arglen, void *template, ...)
{
	va_list ap;
	char *q = malloc(strlen(template) + arglen);

	if (!q)
	{
		dolog(LOG_ERR, "Couldn't allocate memory for query\n");
		exit(-1);
	}
	va_start(ap, template);
	vsnprintf(q, strlen(template) + arglen, template, ap);
	va_end(ap);

	D(dolog(LOG_DEBUG, "Query(): %s\n", q);)
	if (mysql_query(radio.db, q) != 0)
	{
		dolog(LOG_ERR, "SQL error: %s : %s\n", mysql_error(radio.db), q);
		exit(-1);
	}
	free(q);
}

/*
* Open and read the configuration file so that we know the stream's title,
* the URL and our password. Return 1 if ok, or 0 if failure.
*/
int ReadConfigFile(void)
{
	FILE *zin;
	char buf[BSIZE];
	char *cptr;
	char *varname, *value;

	if (radio.c_ConfigFile) zin = fopen(radio.c_ConfigFile, "r");
	else zin = fopen("etc/radio.conf", "r");
	if (zin == NULL) return (0);

	radio.c_MTU = 1200;
	
	while (!feof(zin))
	{
		cptr = fgets(buf, BSIZE - 2, zin);
		if (cptr == NULL) break;

		// Ignore comment lines	
		if (	buf[0] == '#' ||
			(buf[0] == '/' || buf[1] == '/') ||
		/* Ignore whitespace */
			buf[0] == '\n') continue;
	
		/* Trim the newline from the end of the string */
		buf[strlen(buf) - 1]= '\0';
	
		/*
		* Find the= sign in the line, and NULL it after setting value to point
		* after it.
		*/
		value = strchr(buf, '=');
		if (value == NULL)
		{
			dolog(LOG_ERR, "No '=' sign on this line in config file:\n\t%s\n", buf);
			return (0);
		}
		*value= '\0'; value++;
	
		/* Walk value up past any leading spaces */
		while ((*value == ' ') || (*value == '\t')) value++;
	
		/* Trim any trailing whitespace off the variable name */
		varname = buf;
		cptr = strpbrk(varname, " \t");
		if (cptr != NULL) *cptr= '\0';
	
		     if (!strcasecmp(varname, "StreamTitle"))
			radio.c_StreamTitle		= strdup(value);

		else if (!strcasecmp(varname, "URL"))
			radio.c_URL			= strdup(value);

		else if (!strcasecmp(varname, "multicast_source"))
			radio.c_Multicast_Source	= strdup(value);

		else if (!strcasecmp(varname, "multicast_address"))
			radio.c_Multicast_Address	= strdup(value);

		else if (!strcasecmp(varname, "multicast_audio_port"))
			radio.c_Multicast_Audio_Port	= strdup(value);

		else if (!strcasecmp(varname, "multicast_title_port"))
			radio.c_Multicast_Title_Port	= strdup(value);

		else if (!strcasecmp(varname, "database_server"))
			radio.c_DB_Server		= strdup(value);

		else if (!strcasecmp(varname, "database_name"))
			radio.c_DB_Name			= strdup(value);

		else if (!strcasecmp(varname, "database_user"))
			radio.c_DB_User			= strdup(value);

		else if (!strcasecmp(varname, "database_password"))
			radio.c_DB_Password		= strdup(value);

		else if (!strcasecmp(varname, "daemonize"))
			radio.c_Daemonize		= (!strcasecmp(value, "no") ? 0 : 1);

		else if (!strcasecmp(varname, "pidfile"))
			radio.c_PIDFile			= strdup(value);

		else if (!strcasecmp(varname, "signalsfile"))
			radio.c_SignalsFile		= strdup(value);

		else if (!strcasecmp(varname, "mtu"))
			radio.c_MTU			= atoi(value);

		else if (!strcasecmp(varname, "hoplimit"))
			radio.c_HopLimit		= atoi(value);

		else if (!strcasecmp(varname, "interfaces"))
		{
			unsigned int i, l = 0;
			radio.interfaces = 1;

			// First count the number of interfaces
			for (i=0;value[i]!='\0';i++)
			{
				if (value[i] == ',') radio.interfaces++;
			}

			// Allocate some memory for it
			if (radio.lines) free(radio.lines);
			radio.lines = malloc(radio.interfaces*sizeof(struct line));
			if (!radio.lines)
			{
				dolog(LOG_ERR, "Couldn't allocate memory for the lines\n");
				return 0;
			}

			memset(radio.lines, 0, radio.interfaces*sizeof(struct line));

			// Set the sockets to -1
			for (l=0;l<radio.interfaces;l++)
			{
				radio.lines[l].Socket = radio.lines[l].TitleSocket = -1;
			}

			// Now reparse and store the values
			for (i=0,l=0;value[i]!='\0' && l < radio.interfaces;i++)
			{
				radio.lines[l].Interface = atoi(&value[i]);
				printf("Added interface %u: %u\n", l, radio.lines[l].Interface);
				l++;

				// skip till next ',' or the '\0'
				while (value[i]!='\0' && value[i]!=',') i++;
			}
		}
	}
	fclose(zin);

	if (	radio.c_StreamTitle == NULL ||
		radio.c_URL == NULL ||
		radio.c_Multicast_Address == NULL ||
		radio.c_Multicast_Audio_Port == NULL ||
		radio.c_Multicast_Title_Port == NULL ||
		radio.c_DB_Server == NULL ||
		radio.c_DB_Name == NULL ||
		radio.c_DB_User == NULL ||
		radio.c_DB_Password == NULL ||
		radio.c_PIDFile == NULL ||
		radio.lines == NULL ||
		radio.c_MTU < 500)
	{
		dolog(LOG_ERR, "Configuration incomplete, aborting.\n");
		return 0;
	}

	return 1;
}

