/* Radio.Unfix :: Gatherer 
 *
 * This task should be cronned every day or so.
 * When run it checks to see if a file is already
 * entered into the database and/or if it changed (filesize)
 * If it has changed or does not exists it inserts/updates
 * the id3 tag information into the database.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <malloc.h>
#include <mysql/mysql.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include "radio.h"

/* Configuration information */
struct Radio    radio;

/* Queries */
#define SONG_FIND_FILENAME	"SELECT song_id, song_filesize FROM songs WHERE song_filename = '%s'"
#define SONG_UPDATE_NOW		"UPDATE songs SET song_lastChecked = now() WHERE song_id = '%d'"
#define SONG_POOL_DELETED	"UPDATE songs SET pool_id = 0 WHERE song_id = '%d'"
#define SONG_UPDATE_FULL	"UPDATE songs SET song_title = '%s', song_artist = '%s', song_filename = '%s', song_filesize = '%d', song_lastchecked = now(), pool_id = '%d' WHERE song_id = '%d'"
#define SONG_INSERT		"INSERT INTO songs (song_title, song_artist, song_filename, song_filesize, song_lastchecked, pool_id) VALUES ('%s', '%s', '%s', '%d', now(), '%d')"

#define POOL_FIND_NAME		"SELECT pool_id, pool_location FROM pools WHERE pool_name = '%s'"

/* Globals */
MYSQL_RES	*res;
MYSQL_ROW	row;
int		fullrefresh = 1;
int		current_pool_id = -1;
char		*current_pool_dir = NULL;

// Dup and quote a string
char *quotestr(const char *str, int max)
{
	int i = 0, j, count = 0;
	char *dup;

	while (str[i] != '\0' && (max == -1 || i < max))
	{
		if (str[i] == '\'') count++;
		i++;
	}

	dup = malloc(strlen(str) + count + 1);
	if (!dup)
	{
		dolog(LOG_ERR, "Couldn't allocate memory for quotestr()\n");
		exit(-1);
	}

	i = 0;
	j = 0;
	while (str[i] != '\0' && (max == -1 || i < max))
	{
		dup[j] = str[i];
		if (str[i] == '\'')
		{
			j++;
			dup[j] = '\'';
		}
		i++;
		j++;
	}
	dup[j] = '\0';

	return dup;
}

void checkfile(const char *filename)
{
	char		*q, *filename_sql, *artist_sql, *title_sql;
	int		i, j, song_id = -1;
	int		filesize = 0, qlen;
	struct stat	stats;

	// Ignore anything not ending in .mp3
	i = strlen(filename);
	if (	filename[i-4] != '.' ||
		(filename[i-3] != 'm' && filename[i-3] != 'M') ||
		(filename[i-2] != 'p' && filename[i-2] != 'P') ||
		filename[i-1] != '3')
	{
		dolog(LOG_INFO, "Ignoring not-an-mp3: %s\n", filename);
		return;
	}

	filename_sql = quotestr(filename, -1);

	qlen = strlen(SONG_FIND_FILENAME) + strlen(filename_sql);

	// Get the file from SQL
	q = malloc(qlen);
	if (!q)
	{
		dolog(LOG_ERR, "No memory for SONG_FIND_FILENAME query\n");
		exit(-1);
	}
	snprintf(q, qlen, SONG_FIND_FILENAME, filename_sql); 

	D(dolog(LOG_DEBUG, "QUERY: %s\n", q);)
	if (	mysql_query(radio.db, q) != 0 ||
		!(res = mysql_store_result(radio.db)))
	{
		dolog(LOG_ERR, "SQL error: %s : %s\n", mysql_error(radio.db), q);
		exit(-1);
	}
	free(q);
	q = NULL;

	// Walk through it
	while ((row = mysql_fetch_row(res)))
	{
		song_id = atoi(row[0]);
		filesize = atoi(row[1]);
	}

	// Cleanup SQL
	mysql_free_result(res);

	// Get the filesize
	memset(&stats, 0, sizeof(stats));
	stat(filename, &stats);

	// Check if the filesize changed
	if (	!fullrefresh &&
		song_id != -1 &&
		filesize == stats.st_size)
	{
		// Tag the song's lastcheck date, nothing changed
		query(	INTEGER_STRING_SIZE,
			SONG_UPDATE_NOW,
			song_id);

		// Skip it ;)
		free(filename_sql);
		return;
	}

	// Use the lastknown (just read) size
	filesize = stats.st_size;

	// Get data from Mp3
	i = strlen(filename);
	// Skip the directories
	while (i != 0 && filename[i] != '/') i--;

	// we want to skip that / :)
	i++;

	// Find the Artist
	j = i;
	while ( filename[i] != '\0' &&
		filename[i] != '-') i++;

	artist_sql = quotestr(&filename[j], i-j-1);

	// Skip the tracknumber
	i++;
	j = i;
	while ( filename[i] != '\0' &&
		filename[i] != '-') i++;

	// Find the title
	i+=2;
	j = i;
	while ( filename[i] != '\0' &&
		filename[i] != '-') i++;
	title_sql = quotestr(&filename[j], i-j-4);

	if (song_id != -1 && filesize != 0)
	{
		// The song is still the same size and exists
		query(	strlen(title_sql) + strlen(artist_sql) + strlen(filename_sql) + (3*INTEGER_STRING_SIZE),
			SONG_UPDATE_FULL,
			title_sql, artist_sql, filename_sql, filesize, current_pool_id, song_id);
	}
	else if (song_id != -1 && filesize == 0)
	{
		// Song doesn't exist, thus move it to the deleted pool
		query(	INTEGER_STRING_SIZE,
			SONG_POOL_DELETED,
		       	song_id);
	}
	else if (filesize != 0)
	{
		query(	strlen(title_sql) + strlen(artist_sql) + strlen(filename_sql) + (2*INTEGER_STRING_SIZE),
			SONG_INSERT,
			title_sql, artist_sql, filename_sql, filesize, current_pool_id);

	}
	else dolog(LOG_INFO, "Skipping empty file: %s\n", filename);

	free(artist_sql);
	free(title_sql);
	free(filename_sql);
}

void walkdir(const char *dirname);

void statfile(char *name)
{
	struct stat statt;
	if (stat(name, &statt) == 0)
	{
		if (S_ISLNK(statt.st_mode)) statfile(name);
		else if (S_ISDIR(statt.st_mode)) walkdir(name);
		else if (S_ISREG(statt.st_mode)) checkfile(name);
		else dolog(LOG_INFO, "Ignoring %s, unknown mode %d\n", name, statt.st_mode);
	}
	else dolog(LOG_INFO, "Unable to stat %s\n", name);
}

void walkdir(const char *dirname)
{
	DIR *dirp;
	struct dirent *dp;

	D(dolog(LOG_DEBUG, "Walkdir() : %s\n", dirname);)

	dirp = opendir(dirname);

	if (!dirp)
	{
		dolog(LOG_ERR, "couldn't open directory: %s\n", dirname);
		exit(-1);
	}
	
	do
	{
		errno = 0;
		if ((dp = readdir(dirp)) != NULL)
		{
			char *name;
			/* Skip parent and current */
			if (	(strcmp(dp->d_name, ".") == 0) ||
				(strcmp(dp->d_name,"..") == 0)) continue;

			// Construct a filename
			name = malloc(strlen(dirname) + strlen(dp->d_name) + 2);
			strcpy(name, dirname);
			if (dirname[strlen(dirname)-1] != '/') strcat(name, "/");
			strcat(name, dp->d_name);
			if (!name)
			{
				dolog(LOG_ERR, "Couldn't allocate memory for concat\n");
				exit(-1);
			}

			// Follow symlinks, let's hope they don't loop ;)
			if (dp->d_type == DT_LNK)
			{
				char linkname[2000];
				if (readlink(name, linkname, sizeof(linkname)) != -1)
				{
					statfile(linkname);
				}
				else dolog(LOG_INFO, "Couldn't readlink() of %s\n", name);
			}

			// Walk directories
			else if (dp->d_type == DT_DIR) walkdir(name);
			// Check files
			else if (dp->d_type == DT_REG) checkfile(name);
			// Stat unknown files
			else if (dp->d_type == DT_UNKNOWN) statfile(name);

			// Free the memory
			free(name);
		}
	}
	while (dp != NULL);
	closedir(dirp);
	return;
}

/* Select the pool based on the name passed in */
void selectpool(char *poolname)
{
	char *poolname_sql = quotestr(poolname, -1);
	char *q;
	int qlen;

	qlen = strlen(POOL_FIND_NAME) + strlen(poolname_sql);

	// Get the file from SQL
	q = malloc(qlen);
	if (!q)
	{
		dolog(LOG_ERR, "No memory for POOL_FIND_NAME query\n");
		exit(-1);
	}
	snprintf(q, qlen, POOL_FIND_NAME, poolname_sql); 

	D(dolog(LOG_DEBUG, "QUERY: %s\n", q);)
	if (mysql_query(radio.db, q) != 0)
	{
		dolog(LOG_ERR, "SQL error: %s : %s\n", mysql_error(radio.db), q);
		exit(-1);
	}

	if (	mysql_field_count(radio.db) == 0 ||
		(!(res = mysql_store_result(radio.db))))
	{
		dolog(LOG_ERR, "No such pool: %s\n", poolname);
		exit(-1);
	}
	free(poolname_sql);
	free(q);
	q = NULL;

	// Walk through it
	while ((row = mysql_fetch_row(res)))
	{
		current_pool_id = atoi(row[0]);
		current_pool_dir = strdup(row[1]);
	}

	// Cleanup SQL
	mysql_free_result(res);
}

int main(int argc, char **argv)
{
	char *zErrMsg = 0;
	
	if ( argc != 2)
	{
		dolog(LOG_ERR, "Usage: %s poolname\n", argv[0]);
		return -1;
	}

	/* Read config file */
	if (ReadConfigFile() != 1)
	{
		dolog(LOG_ERR, "Bad or missing config file at %s, giving up\n", radio.c_ConfigFile);
		return -1;
	}

	// We won't want this to daemonize (at least I don't)
	radio.c_Daemonize = 0;

	radio.db = mysql_init(NULL);
	if (!radio.db)
	{
		dolog(LOG_ERR, "Couldn't initialize MySQL\n");
		return -1;
	}

	if (!mysql_real_connect(
		radio.db,
		radio.c_DB_Server,
		radio.c_DB_Name,
		radio.c_DB_User,
		radio.c_DB_Password,
		0, NULL, 0))
	{
		dolog(LOG_ERR, "Couldn't open database\n");
		return -1;
	}

	selectpool(argv[1]);

	if (current_pool_id == -1)
	{
		dolog(LOG_ERR, "I require a valid pool first\n");
		return -1;
	}

	if (current_pool_id == 0)
	{
		dolog(LOG_ERR, "I cannot gather the deleted pool ;)\n");
		return -1;
	}

	walkdir(current_pool_dir);

	// Be nice, free the memory
	free(current_pool_dir);

	// Close the database
	mysql_close(radio.db);

	return 0;
}

